#ifndef _CAMERA_H
#define _CAMERA_H

#include <math.h>

#define PI 3.14159265358979323846

#define toRadians(d) d*PI/180

vec3& Normalize(vec3& v){
	double ilen = 1.0/v.length();
	v.x *= ilen;
	v.y *= ilen;
	v.z *= ilen;
	return v;
}

vec3 Cross(const vec3& v1, const vec3& v2){
	double x = v1.y*v2.z-v1.z*v2.y;
	double y = v1.z*v2.x-v1.x*v2.z;
	double z = v1.x*v2.y-v1.y*v2.x;
	return vec3(x, y, z);
}

class Camera {
private:
	double tx, ty, tz;
	double r, theta, phi, x, y, z, up;
	void calcCoords(){
		if(r < 1) r = 1;
		x = r*cos(theta)*sin(phi)+tx;
		y = r*sin(theta)*sin(phi)+ty;
		z = r*cos(phi)+tz;
		up = (sin(phi) > 0)?1.0:-1.0;
	}
public:
	Camera(){
		tx = 1500; ty = 100; tz = 10;
		r = 1; theta = toRadians(-90); phi = 1.55650964; up = 1;
		calcCoords();
		button = -1;
	}
	void setPos(double tx, double ty, double tz){x = tx; y = ty; z = tz;}
	void setTarget(double x, double y, double z){tx = x; ty = y; tz = z;}
	void loadCamera(){gluLookAt(x, y, z, tx, ty, tz, 0, 0, up);}
	void loadCameraPos(Shader* s, const char* label){s->set3f(label, (float)x, (float)y, (float)z);}
	vec3 getCameraDir(){return vec3(tx-x, ty-y, tz-z);}
	vec3 getCameraPos(){return vec3(x, y, z);}

private:
	int button, oldX, oldY;
public:

	void keyPress( int key, int x, int y ){
		vec3 v = getCameraDir();
		Normalize(v);
		v = v*2;
		switch(key){
		case GLUT_KEY_LEFT: v = Cross(v, vec3(0, 0, up)); tx -= v.x; ty -= v.y; tz -= v.z; calcCoords(); glutPostRedisplay(); break;
		case GLUT_KEY_RIGHT: v = Cross(v, vec3(0, 0, up)); tx += v.x; ty += v.y; tz += v.z; calcCoords(); glutPostRedisplay(); break;
		case GLUT_KEY_UP: tx += v.x; ty += v.y; tz += v.z; calcCoords(); glutPostRedisplay(); break;
		case GLUT_KEY_DOWN: tx -= v.x; ty -= v.y; tz -= v.z; calcCoords(); glutPostRedisplay(); break;
		}
	}
	void mouse( int b, int state, int x, int y ){
		button = -1;
		if (state == GLUT_DOWN){
			button = b;
			oldX = x;
			oldY = y;
		}
	}
	void dragged( int x, int y ){
		if(button == GLUT_RIGHT_BUTTON){
			int dy = y-oldY;
			r += dy;
			calcCoords();
			glutPostRedisplay();
		} else if(button == GLUT_LEFT_BUTTON) {
			double dx = -(x-oldX)*0.01;
			double dy = -(y-oldY)*0.01;
			theta += dx;
			phi += dy;
			calcCoords();
			glutPostRedisplay();
		}
		oldX = x;
		oldY = y;
	}
};

#endif