#ifndef _SHADER_H
#define _SHADER_H

#include <string.h>
#include <stdio.h>

class Shader {
private:
	uint id;
	char* loadFile(const char* file){
		FILE *f = fopen(file, "rb");
		if (f == NULL){printf("Error: Unable to locate shader files."); exit(-1); return NULL;}
		fseek(f, 0, SEEK_END);
		long size = ftell(f);
		fseek(f, 0, SEEK_SET);
		char* ret = new char[size+1];
		fread(ret, size, 1, f);
		fclose(f);
		ret[size] = '\0';
		return ret;
	}
	uint compileShader(const char* s, uint type){
		uint id = glCreateShader(type);
		glShaderSource(id, 1, &s, NULL);
		glCompileShader(id);
		int c = 0;
		glGetShaderiv(id, GL_COMPILE_STATUS, &c);
		if(!c){
			char* logstr = new char[256];
			glGetShaderInfoLog(id, 256, NULL, logstr);
			printf("Error compiling shader %s:\n%s\n", s, logstr);
			delete logstr;
			exit(-1);
			return 0;
		}
		return id;
	}
public:
	Shader(const char* file){
		char ftmp[256];
		strcpy(ftmp, file);
		strcat(ftmp, ".vert");
		char* v = loadFile(ftmp);
		uint vert = compileShader(v, GL_VERTEX_SHADER);
		delete[] v;
		strcpy(ftmp, file);
		strcat(ftmp, ".frag");
		char* f = loadFile(ftmp);
		uint frag = compileShader(f, GL_FRAGMENT_SHADER);
		delete[] f;
		id = glCreateProgram();
		glAttachShader(id, vert);
		glAttachShader(id, frag);
		glLinkProgram(id);
	}
	Shader(const char* v, const char* f){
		uint vert = compileShader(v, GL_VERTEX_SHADER);
		uint frag = compileShader(f, GL_FRAGMENT_SHADER);
		id = glCreateProgram();
		glAttachShader(id, vert);
		glAttachShader(id, frag);
		glLinkProgram(id);
	}
	~Shader(){
		glDeleteShader(id);
	}
	void bind(){glUseProgram(id);}
	void unbind(){glUseProgram(0);}
	void seti(const char* label, int arg){
		bind();
		glUniform1iARB(glGetUniformLocationARB(id, label), arg);
	}
	void setf(const char* label, float arg){
		bind();
		glUniform1fARB(glGetUniformLocationARB(id, label), arg);
	}
	void set2i(const char* label, int arg1, int arg2){
		bind();
		glUniform2iARB(glGetUniformLocationARB(id, label), arg1, arg2);
	}
	void set2f(const char* label, float arg1, float arg2){
		bind();
		glUniform2fARB(glGetUniformLocationARB(id, label), arg1, arg2);
	}
	void set3i(const char* label, int arg1, int arg2, int arg3){
		bind();
		glUniform3iARB(glGetUniformLocationARB(id, label), arg1, arg2, arg3);
	}
	void set3f(const char* label, float arg1, float arg2, float arg3){
		bind();
		glUniform3fARB(glGetUniformLocationARB(id, label), arg1, arg2, arg3);
	}
	void set4i(const char* label, int arg1, int arg2, int arg3, int arg4){
		bind();
		glUniform4iARB(glGetUniformLocationARB(id, label), arg1, arg2, arg3, arg4);
	}
	void set4f(const char* label, float arg1, float arg2, float arg3, float arg4){
		bind();
		glUniform4fARB(glGetUniformLocationARB(id, label), arg1, arg2, arg3, arg4);
	}
	void set3iv(const char* label, const int* args){
		bind();
		glUniform3ivARB(glGetUniformLocationARB(id, label), 1, args);
	}
	void set3fv(const char* label, const float* args){
		bind();
		glUniform3fvARB(glGetUniformLocationARB(id, label), 1, args);
	}
	void setMatrix(const char* label, const float* m){
		bind();
		glUniformMatrix4fvARB(glGetUniformLocationARB(id, label), 1, false, m);
	}
	void setMatrix(const char* label, const double* m){
		bind();
		float fm[] = {(float)m[0], (float)m[1], (float)m[2], (float)m[3], (float)m[4], (float)m[5], (float)m[6], (float)m[7], (float)m[8], (float)m[9], (float)m[10], (float)m[11], (float)m[12], (float)m[13], (float)m[14], (float)m[15]};
		glUniformMatrix4fvARB(glGetUniformLocationARB(id, label), 1, false, fm);
	}
};

#endif